@@ -1,7 +1,7 @@
-id = "45c9dd9d16bc677a761b629ab559a22d24950f30"
-date = "2014-11-19 22:12:50 +0000"
+id = "5fd44d9165c1083e35c615f1eb95979a07e0e3d0"
+date = "2015-05-22 20:41:05 +0000"
 branch = "None"
-tag = "glue-release-1-47-1"
+tag = "glue-release-1-48"
 if tag == "None":
     tag = None
 author = "Ryan Fisher <ryan.fisher@ligo.org>"
@@ -10,11 +10,11 @@ committer = "Ryan Fisher <ryan.fisher@ligo.org>"
 status = "CLEAN: All modifications committed"
 version = id
 verbose_msg = """Branch: None
-Tag: glue-release-1-47-1
-Id: 45c9dd9d16bc677a761b629ab559a22d24950f30
+Tag: glue-release-1-48
+Id: 5fd44d9165c1083e35c615f1eb95979a07e0e3d0
 
 Builder: Ryan Fisher <ryan.fisher@ligo.org>
-Build date: 2014-11-19 22:14:15 +0000
+Build date: 2015-05-22 20:42:30 +0000
 Repository status: CLEAN: All modifications committed"""
 
 import warnings
@@ -33,9 +33,9 @@ def check_match(foreign_id, onmismatch="raise"):
     """
     if onmismatch not in ("raise", "warn"):
         raise ValueError, onmismatch + " is an unrecognized value of onmismatch"
-    if foreign_id == "45c9dd9d16bc677a761b629ab559a22d24950f30":
+    if foreign_id == "5fd44d9165c1083e35c615f1eb95979a07e0e3d0":
         return
-    msg = "Program id (45c9dd9d16bc677a761b629ab559a22d24950f30) does not match given id (%s)." % foreign_id
+    msg = "Program id (5fd44d9165c1083e35c615f1eb95979a07e0e3d0) does not match given id (%s)." % foreign_id
     if onmismatch == "raise":
         raise VersionMismatchError, msg
 
