@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2014  Kipp Cannon
+# Copyright (C) 2007-2015  Kipp Cannon
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -1028,8 +1028,7 @@ def use_in(ContentHandler):
 	...		super(MyContentHandler, self).__init__(*args)
 	...		self.connection = sqlite3.connection()
 	...
-	>>> from glue.ligolw import dbtables
-	>>> dbtables.use_in(MyContentHandler)
+	>>> use_in(MyContentHandler)
 
 	Multiple database files can be in use at once by creating a content
 	handler class for each one.
